"""
    (c) 2021 Rodney Maniego Jr.
    Namari
"""

import hashlib
from arkivist import Arkivist


class Namari():
    def __init__(self, filepath=None, autosave=False):
        self.filepath = _validate_filepath(filepath)
        self.data = Arkivist(self.filepath, autosave=autosave)
    
    def insert(self, elements):
        if type(elements) not in (list, set, tuple):
            elements = [elements]
        for element in elements:
            if _validate_key(element):
                self.data.set(element, [])
        return self
    
    def set(self, parent, child):
        if _validate_key(parent) and _validate_key(child):
            self.data.appendIn(parent, child)
        return self
    
    def attach(self, parent, child):
        if _validate_key(parent) and _validate_key(child):
            if parent in self.data:
                self.data.appendIn(parent, child)
        return self
    
    def detach(self, parent, child):
        if _validate_key(parent) and _validate_key(child):
            if parent in self.data:
                self.data.removeIn(parent, child)
        return self
    
    def get(self, parent, fallback=None):
        if _validate_key(parent):
            return self.data.get(parent, fallback)
        return fallback
    
    def findFirst(self, search, fallback=None):
        if _validate_key(search):
            for parent, children in self.data.items():
                if search in children:
                    return parent
        return fallback
    
    def findAll(self, search):
        matches = []
        if _validate_key(search):
            for parent, children in self.data.items():
                if search in children:
                    matches.append(parent)
        return matches
    
    def items(self):
        for parent, children in self.data.items():
            yield (parent, children)
    
    def keys(self):
        return list(self.data.keys())
    
    def values(self):
        return list(self.data.values())
    
    def contains(self, search):
        if _validate_key(search):
            if (search in self.data):
                return True
            for children in self.data.values():
                if search in children:
                    return True
        return False
    
    def count(self):
        return len(self.data)
    
    def is_empty(self):
        return self.data.is_empty()

    def clear(self):
        self.data.reset()
        

def _validate_key(key):
    return type(key) in (str, int, float)

def _validate_filepath(filepath):
    if isinstance(filepath, str):
        if filepath.split(".")[-1] != "json":
            filepath += ".json"
        try:
            with open(filepath, "a+") as temp:
                return filepath
        except:
            pass
    return None


from namari import Namari

alphabet = "abcdefghijklmnopqrstuvwxyz "
lorem = "Lorem ipsum dolor sit amet, consectetur adipiscing elit. Vestibulum elementum mauris mattis, varius lectus eu, mattis neque. Nam faucibus posuere mi, ac malesuada tortor dictum egestas. Aenean et ipsum est. Morbi et nisl felis. In maximus ultricies ex, quis interdum massa maximus eget. Suspendisse efficitur vel sapien id tincidunt. Aenean hendrerit, urna non ornare suscipit, dui magna luctus justo, sit amet gravida sem quam a dolor. Praesent posuere non massa non lobortis. Sed elementum id turpis quis tempor. Proin vestibulum magna laoreet nisl facilisis finibus. Vivamus ut ornare sem."
lorem = "".join([x for x in lorem if x.lower() in alphabet])

lexicon = Namari("test.namari.json")

print("\n\n# of items:", lexicon.count())
print("\nClearing items...")
lexicon.clear()

if lexicon.is_empty():
    print("\n\nPopulating dataset...")
    for word in lorem.split(" "):
        lexicon.set(word, word)
        lexicon.attach(word, 0)
        lexicon.attach(word, word.lower())
        lexicon.attach(word, word.upper())
        lexicon.attach(word, word.title())
        lexicon.attach(word, word.capitalize())
        lexicon.detach(word, 0)
        count = lexicon.count()
        item = lexicon.findFirst(word.lower(), "hello, world")
        print(f"#{count} {item}")

print("\n[Get Children]")
print("Hello:", lexicon.get("Hello"))
print("Lorem:", lexicon.get("Lorem"))


print("\n[Get Parents]")
lexicon.attach("Lorem", "ipsum")
print("Parents: ", lexicon.findAll("ipsum"))

print("\n[check]")
print("Contains \"lorem\":", lexicon.contains("lorem"))
print("Contains \"hello\":", lexicon.contains("hello"))


keys = [str(x) for x in lexicon.keys()]
print("\nKeys:", ", ".join(keys))

print("\nValues: ", lexicon.values())


print("\n\n[dataset]")
for keys, value in lexicon.items():
    print(f" - {keys}: {value}")

print("\n# of items:", lexicon.count())


codes = Namari()
for x in range(100):
    if not codes.contains(f"#{x}"):
        codes.insert(f"#{x}")
    if x > 0:
        codes.set(f"#{x}", f"#{x-1}")
    codes.attach(f"#{x}", f"#{x}")

for key, value in codes.items():
    print(key, value)


import setuptools

with open("README.md", "r") as f:
    long_description = f.read()

setuptools.setup(
    name = "namari",
    packages = ["namari"],
    version = "1.1.7",
    license="MIT",
    description = "Many-to-one keys-value pair relationship Python object manager.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author = "Rodney Maniego Jr.",
    author_email = "rod.maniego23@gmail.com",
    url = "https://github.com/rmaniego/namari",
    download_url = "https://github.com/rmaniego/namari/archive/v1.0.tar.gz",
    keywords = ["Dictionary", "JSON", "many-to-many", "keys-value", "multiple relationship"],
    install_requires=["arkivist"],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers", 
        "Topic :: Software Development :: Build Tools",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3"
    ],
    python_requires=">=3.6"
)